import { CreateProperties, ExistingProperties, GetByProperty, QueryConfig } from "../../src/types/profiles";
import { FilteredResponse } from "../../src/types/sdkResponse";
import { QueryParams } from "../../src/types/queryBuilder";
/**
 * @function get
 * @param {string} baseUrl
 * @param {Record<string, string>} headers
 * @param {string} profileId
 * @returns {FilteredResponse}
 */
export declare function get(baseUrl: string, headers: Record<string, string>, profileId: string): FilteredResponse;
/**
 * @function create
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @param {CreateProperties} properties
 * @returns {FilteredResponse}
 */
export declare function create(baseUrl: string, auth: Record<string, string>, properties: CreateProperties): FilteredResponse;
/**
 * @function delete
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @param {string} profileId
 * @returns {FilteredResponse}
 */
export declare function deleteProfile(baseUrl: string, auth: Record<string, string>, profileId: string): FilteredResponse;
/**
 * @function count
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @returns {FilteredResponse}
 */
export declare function count(baseUrl: string, auth: Record<string, string>): FilteredResponse;
/**
 * @function existingProperties
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @param {ExistingProperties} params
 * @returns {FilteredResponse}
 */
export declare function existingProperties(baseUrl: string, auth: Record<string, string>, params: ExistingProperties): FilteredResponse;
/**
 * @function allProperties
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @returns {FilteredResponse}
 */
export declare function allProperties(baseUrl: string, auth: Record<string, string>): FilteredResponse;
/**
 * @function sessions
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @param {FilteredResponse} profileId
 * @returns {FilteredResponse}
 */
export declare function sessions(baseUrl: string, auth: Record<string, string>, profileId: string): FilteredResponse;
/**
 * @function getByProperty
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @param {GetByProperty} params
 * @returns {FilteredResponse}
 */
export declare function getBySingleProperty(baseUrl: string, auth: Record<string, string>, params: GetByProperty): FilteredResponse;
/**
 * @function query
 * @param {string} baseUrl
 * @param {Record<string, string>} auth
 * @param {GetByProperty} params
 * @param {QueryParams[]} query
 * @returns {FilteredResponse}
 */
export declare function query(baseUrl: string, auth: Record<string, string>, params: QueryConfig, query: QueryParams[]): FilteredResponse;
/**
 * @function allUsedProperties
 * @param {string} baseUrl
 * @param {Record<string, string>} headers
 * @returns {FilteredResponse}
 */
export declare function allUsedProperties(baseUrl: string, headers: Record<string, string>): FilteredResponse;
